'From Smalltalk/X, Version:5.2.8 on 15-01-2008 at 00:21:20'                     !

"{ Package: 'SpreadSheet:plickm1' }"

AbstractTable subclass:#ArrayedTable
	instanceVariableNames:'rows columnsCount'
	classVariableNames:''
	poolDictionaries:''
	category:'obp-SpreadSheet'
!

!ArrayedTable methodsFor:'accessing'!

cellAtRow: rowIndex column: colIndex
        "return cell at given address"

        ^ (self getRow: rowIndex) cellAt: colIndex.
!

cellAtRow: rowIndex column: colIndex put: object
        "set cell at given address"
        (self allocateRow: rowIndex) cellAt: colIndex put: object.

        self updateColumnsCount: colIndex.
!

columnAt:colIndex 
    |tmp|

    tmp := Array new:(self rowsCount).

    1 to: (self rowsCount) do: [:x| tmp at:x put: (self cellAtRow: x column: colIndex)].
    ^ tmp.
!

columnAt:colIndex put:array 
    "set given array as column with given index (rewrites current)"
    
    1 to:(array size) do:[:x | 
        self 
            cellAtRow:x
            column:colIndex
            put:(array at:x)
    ].

    (array size + 1) to:(self rowsCount) do:[:x | 
        self 
            cellAtRow:x
            column:colIndex
            put:nil
    ].
!

emptyCellAtRow:rowIndex column:colIndex 
    "deletes content of cell at given address"
    
    (self getRow:rowIndex) emptyCellAt:colIndex.
!

emptyColumnAt: colIndex
   "empties whole column with given index"
   self rows do: [:x| 
        x ifNotNil: [
           x emptyCellAt: colIndex.
        ]
   ]
!

emptyRowAt:rowIndex 
    "empties whole row with given index"
    
    (rowIndex > (self rows size)) ifFalse:[
        self rows at:rowIndex put:nil.
    ].
!

rowAt: rowIndex
   "returns array containing whole row with given index"
   |tmp array|
   tmp := (self getRow: rowIndex) asArray.

   array:= Array new: (self columnsCount).
   array replaceFrom: 1 to: (tmp size) with: tmp startingAt:1.
   ^ array.
!

rowAt:rowIndex put:array 
    "set given array as row with given index (rewrites current)"
    
    1 to:(array size) do:[:x | 
        self 
            cellAtRow:rowIndex
            column:x
            put:(array at:x)
    ].

     "possible optimalization here"
    (array size + 1) to:(self columnsCount) do:[:x | 
        self 
            cellAtRow:rowIndex
            column:x
            put:nil
    ].
! !

!ArrayedTable methodsFor:'iteration'!

columnsDo:block 
    "runs given block with all columns giving actual columns as a parameter"
    1 to: (self columnsCount) do: [:x|
      block value: (self columnAt: x).
    ]
!

do: block
   "runs given block with all cells (row by row) giving actual cell as a parameter"
   self rowsDo: [:x|
      x do: [:y|
         block value: y.
      ]
   ]
!

rowsDo:block 
    "runs given block with all rows giving actual row as a parameter"
    
    1 to:(self rowsCount) do:[:x | 
        block value:(self rowAt:x).
    ]
! !

!ArrayedTable methodsFor:'modifying'!

delete:number columnsAt:colIndex 
    "delete number of columns at position given by colIndex (move ascending columns to the left) "
    
    columnsCount := 0.
    self rows do:[:x | 
        x 
            ifNotNil:[
                x delete:number cellsAt:colIndex.
                self updateColumnsCount:(x cells size).
            ].
    ]
!

delete:number rowsAt:rowIndex 
    "delete number of rows at position given by rowIndex (move ascending rows up)"
    
    (rowIndex <= (self rows size)) ifTrue:[
        |maxIndex|

        maxIndex := (rowIndex + number - 1) min:(self rows size).
        self rows removeFromIndex:rowIndex toIndex:maxIndex.
    ]
!

insert:number emptyColumnsAt:colIndex 
    "inserts number of empty columnts at position given by colIndex"
    
    self rows do:[:x | 
        x 
            ifNotNil:[
                x insert:number emptyCellsAt:colIndex.
                self updateColumnsCount:(x cells size).
            ].
    ].
!

insert:number emptyRowsAt:rowIndex 
    "inserts number of empty rows at position given by rowIndex"
    
    (rowIndex <= (self rows size)) ifTrue:[
        self rows addAll:(Array new:number) beforeIndex:rowIndex.
    ].
!

insertColumn:anArray at:colIndex 
    "inserts anArray as a column at colIndex (moves remaining columns to the right"
    
    self insert: 1 emptyColumnsAt: colIndex.
    self columnAt: colIndex put: anArray.
!

insertRow:anArray at:rowIndex 
    "inserts anArray as a row at rowIndex (moves remaining rows to the bottom"
    
    self insert: 1 emptyRowsAt:rowIndex.
    self rowAt:rowIndex put:anArray.
! !

!ArrayedTable methodsFor:'private-implementation'!

allocateRow: rowIndex
        "returns existing or empty (when none) row"

        self rows changeCapacityTo: rowIndex.
        [rowIndex > self rows size] whileTrue: [
            self rows add: nil.
        ].


        (self rows at: rowIndex) ifNil: [
               self rows at: rowIndex put: (ArrayedTableRow new).
        ].

        ^ self rows at: rowIndex.
!

getRow:rowIndex 
    (rowIndex > (self rows size)) ifTrue:[
        ^ (ArrayedTableRow new).
    ] ifFalse:[
        (self rows at:rowIndex) 
            ifNil:[ ^ (ArrayedTableRow new) ]
            ifNotNil:[ ^ self rows at:rowIndex ].
    ].
!

rows
        "return rows array"
        rows ifNil: [
                rows:= OrderedCollection new:0.
        ].

        ^ rows.
!

updateColumnsCount: columnIndex
        "must be called after inserting new item"
        (columnIndex > self columnsCount) ifTrue: [
                columnsCount := columnIndex.
         ].
! !

!ArrayedTable methodsFor:'querying'!

columnsCount
   "returns number of columns"
   columnsCount ifNil: [columnsCount := 0].
   ^ columnsCount.
!

rowsCount
   "returns number of rows"
   ^ self rows size.
! !

