'From Smalltalk/X, Version:5.2.8 on 15-01-2008 at 00:21:20'                     !

"{ Package: 'SpreadSheet:plickm1' }"

TestCase subclass:#CellReferenceTests
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'obp-SpreadSheet'
!

!CellReferenceTests methodsFor:'test'!

testRecursion
    |sheet pravda|

    sheet := SpreadSheet new.
    sheet 
        cellAtRow:1
        column:1
        put:(((13))).
    sheet 
        cellAtRow:1
        column:2
        put:(CellReference 
                newAtRow:1
                column:1
                atSheet:sheet).
     "Reference OK"
    self assert:(((sheet cellAtRow:1 column:2) value) = 13).


    sheet 
        cellAtRow:1
        column:3
        put:(CellReference 
                newAtRow:1
                column:4
                atSheet:sheet).
    sheet 
        cellAtRow:1
        column:4
        put:(CellReference 
                newAtRow:1
                column:3
                atSheet:sheet).
     "reference cycle with CellReference on top"
    [
        pravda := false.
        ((sheet cellAtRow:1 column:4) value).
    ] on:RecursiveDefinitionError do:[ pravda := true. ].
    self assert:(pravda).
    sheet 
        cellAtRow:1
        column:4
        put:(CellExpression with:(CellReference 
                        newAtRow:1
                        column:3
                        atSheet:sheet)).

    " reference cycle with CellExpression top, cycle through CellReference"
    [
        pravda := false.
        ((sheet cellAtRow:1 column:4) value).
    ] on:RecursiveDefinitionError do:[ pravda := true. ].
    self assert:(pravda).


    sheet 
        cellAtRow:1
        column:4
        put:(CellExpression 
                with:(ExprFunSum with:(CellReferenceRange 
                                newFromR:1
                                C:2
                                toR:1
                                C:3
                                atSheet:sheet))).

     "reference cycle through CellReferenceRange"
    [
        pravda := false.
        ((sheet cellAtRow:1 column:4) value).
    ] on:RecursiveDefinitionError do:[ pravda := true. ].
    self assert:(pravda).
!

testReference
   |sheet sheet2 ref|
   sheet := SpreadSheet new.
   sheet appendRow: #(1 2).
   sheet2 := SpreadSheet new.
   sheet2 appendRow: #(3 4).

   ref:= CellReference new.

   ref setReferenceRow: 1 column: 1 atSheet: sheet.
   self assert: ((ref value) = 1).

   ref setReferenceRow: 1 column: 2 atSheet: sheet2.
   self assert: ((ref value) = 4).

   ref setReferenceRow: 2 column: 2 atSheet: sheet2.
   self assert: ((ref value) = nil).
!

testReferenceRange
    |sheet ref str|

    sheet := SpreadSheet new.
    sheet appendRow:#( 1 2 3 4 5 ).
    sheet appendRow:#( 11 12 13 14 15 ).
    ref := CellReferenceRange new.
    ref 
        setReferenceFromR:1
        C:3
        toR:2
        C:4
        atSheet:sheet.
    self assert:(ref value = 3).

     "converts values to TopLeft + BottomRight combination automaticaly"
    ref 
        setReferenceFromR:2
        C:3
        toR:1
        C:4
        atSheet:sheet.
    self assert:(ref value = 3).

     "assert number of listeners of sheet still to be 1"
    self assert:(sheet reindexListeners size = 1).

     "iterating all cells"
    str := ''.
    ref do:[:x | 
        str := str , x asString.
    ].
    self assert:(str = '341314').

     "iterating all rows"
    str := ''.
    ref 
        rowsDo:[:x | 
            x do:[:y | 
                str := str , y asString.
            ].
        ].
    self assert:(str = '341314').
!

testSheetListeners
   |sheet sheet2 ref|
   sheet := SpreadSheet new.
   sheet2 := SpreadSheet new.
   ref:= CellReference new.

   ref setReferenceRow: 1 column: 1 atSheet: sheet.
   self assert: ((sheet reindexListeners size)=1).

   ref setReferenceRow: 1 column: 1 atSheet: sheet2.
   "listener moved from sheet to sheet2"
   self assert: ((sheet reindexListeners size) = 0).
   self assert: ((sheet2 reindexListeners size) = 1).
!

testSheetReindexing
    |sheet ref str|

    sheet := SpreadSheet new.
    ref := CellReference new.
    sheet appendRow:#( 1 2 3 ).
    sheet appendRow:#( 4 5 6 ).
    ref 
        setReferenceRow:2
        column:2
        atSheet:sheet.
    sheet insertRow:#( 9 9 9 ) at:2.
    self assert:(ref value = 5).
    sheet insertColumn:#( 8 8 8 ) at:1.
    self assert:(ref value = 5).

     "reindexing range"
    sheet := SpreadSheet new.
    ref := CellReferenceRange new.
    sheet appendRow:#( 1 2 3 ).
    sheet appendRow:#( 4 5 6 ).
    ref 
        setReferenceFromR:1
        C:2
        toR:2
        C:3
        atSheet:sheet.
    sheet insertRow:#( 9 9 9 ) at:2.
    sheet insertRow: #( 8 8 8) at:1.
    sheet insertRow: #(7 7 7) at:5 .

     "iterating all cells"
    str := ''.
    ref do:[:x | 
        str := str , x asString.
    ].
    self assert:(str = '239956').
     "iterating all rows"
    str := ''.
    ref 
        rowsDo:[:x | 
            x do:[:y | 
                str := str , y asString.
            ].
        ].
    self assert:(str = '239956').
! !

