'From Smalltalk/X, Version:5.2.8 on 15-01-2008 at 00:21:20'                     !

"{ Package: 'SpreadSheet:plickm1' }"

Object subclass:#SpreadSheet
	instanceVariableNames:'data reindexListeners name'
	classVariableNames:''
	poolDictionaries:''
	category:'obp-SpreadSheet'
!

SpreadSheet class instanceVariableNames:'number'

"
 No other class instance variables are inherited by this class.
"
!

!SpreadSheet class methodsFor:'instance-creation'!

new
   |tmp|
   tmp:= super new.
   self numberInc.
   tmp name: ('Sheet',(self number printStringRadix:10 size:2 fill:$0)).
   ^ tmp.
! !

!SpreadSheet class methodsFor:'private'!

number
   number ifNil: [number:= 0.].
   ^number.
!

numberInc
   number ifNil: [number:= 0.].
   number := (number + 1).
! !

!SpreadSheet methodsFor:'accessing'!

cellAtRow:rowIndex column:colIndex 
    "return cell at given address"
    
    ^ self data cellAtRow:rowIndex column:colIndex
!

cellAtRow:rowIndex column:colIndex put:anObject 
    "set cell at given address"
    
    self data 
        cellAtRow:rowIndex
        column:colIndex
        put:anObject
!

columnAt:colIndex 
    "returns array containing column at given colIndex"
    
    ^ self data columnAt:colIndex
!

columnAt:colIndex put:anArray 
    "set given array as column with colIndex (rewrites current column)"
    
    self data columnAt:colIndex put:anArray
!

data
    "return collection containing data"
    
    data ifNil:[ data := ArrayedTable new. ].
    ^ data.
!

emptyCellAtRow:rowIndex column:colIndex 
    "empties cell (sets to nil) at given address"
    
    self data emptyCellAtRow:rowIndex column:colIndex.
!

emptyColumnAt:colIndex 
    "empty whole column (fill it with nils)"
    
    self data emptyColumnAt:colIndex.
!

emptyRowAt:rowIndex 
    "empty whole row (fill it with nils)"
    
    self data emptyRowAt:rowIndex.
!

rowAt:rowIndex 
    "returns array containing whole row"
    
    ^ self data rowAt:rowIndex
!

rowAt:rowIndex put:anArray 
    "sets given array as row at rowIndex (rewrites current row)"
    
    self data rowAt:rowIndex put:anArray.
! !

!SpreadSheet methodsFor:'iteration'!

columnsDo:aBlock 
    "Executes aBlock with every column passing it as an array in parameter"
    
    self data columnsDo:aBlock
!

do:aBlock 
    "Executes aBlock with every cell passing it as a parameter"
    
    self data do:aBlock
!

rowsDo:aBlock 
    "Executes aBlock with every row passing it as an array in parameter"
    
    self data rowsDo:aBlock
! !

!SpreadSheet methodsFor:'listen'!

notifyColumnsFrom:colIndex reindexedBy:aNumber 
    "notifies all listeners that all columns with index >= colIndex 
     got new index increased by aNumber"
    
    self reindexListeners do:[:x | 
        x ifNotNil:[
                x notifyColumnsFrom:colIndex reindexedBy:aNumber
        ].
    ]
!

notifyRowsFrom:rowIndex reindexedBy:aNumber 
    "notifies all listeners that all rows with index >= rowIndex 
     got new index increased by aNumber"
    
    self reindexListeners do:[:x | 
        x ifNotNil:[
                x notifyRowsFrom:rowIndex reindexedBy:aNumber
        ].
    ]
!

registerReindexListener:anObject 
    "Adds listener that needs to be informed about reindexing (after inserting cells)"
    
    self reindexListeners add:anObject.
!

unregisterReindexListener:anObject 
    "removes listener"
    
    self reindexListeners remove:anObject.
! !

!SpreadSheet methodsFor:'modifying'!

appendColumn:anArray 
    "appends given array as a column to the end of table, returns its index"
    
    self columnAt:(self columnsCount + 1) put:anArray.
    ^ self columnsCount.
!

appendRow:anArray 
    "appends given array as a row to the end of table, returns its index"
    
    self rowAt:(self rowsCount + 1) put:anArray.
    ^ self rowsCount.
!

delete:number columnsAt:colIndex 
    "deletes number of columns starting at colIndex"
    
    self data delete:number columnsAt:colIndex.
    self notifyColumnsFrom: (colIndex+number) reindexedBy: ( 0 - number).
!

delete:number rowsAt:rowIndex 
    "deletes number of rows starting at rowIndex"
    
    self data delete:number rowsAt:rowIndex.
    self notifyRowsFrom: (rowIndex+number) reindexedBy: ( 0 - number).
!

deleteColumnAt:colIndex 
    "deletes column at colIndex (moves remaining columns to the left"
    
    self delete: 1 columnsAt:colIndex
!

deleteRowAt:rowIndex 
    "deletes row at rowIndex (moves remaining row upwards"
    
    self delete: 1 rowsAt:rowIndex
!

insert:number emptyColumnsAt:colIndex 
    "inserts number of empty (nils) columns at given index (moves remaining columns to the right"
    
    self data insert:number emptyColumnsAt:colIndex.
    self notifyColumnsFrom: colIndex reindexedBy: number.
!

insert:number emptyRowsAt:rowIndex 
    "inserts number of empty (nils) rows at given index (moves remaining rows to the bottom)"
    
    self data insert:number emptyRowsAt:rowIndex.
    self notifyRowsFrom: rowIndex reindexedBy: number.
!

insertColumn:anArray at:colIndex 
    "inserts anArray as a column at colIndex (moves remaining columns to the right)"
    
    self data insertColumn:anArray at:colIndex.
    self notifyColumnsFrom: colIndex reindexedBy: 1.
!

insertEmptyColumnAt:colIndex 
    "inserts empty (nils) column at given index (moves remaining columns to the right)"
    
    self insert:1 emptyColumnsAt:colIndex.
!

insertEmptyRowAt:rowIndex 
    "inserts empty (nils) row at given index (moves remaining rows to the bottom)"
    
    self insert: 1 emptyRowsAt:rowIndex.
!

insertRow:anArray at:rowIndex 
    "inserts anArray as a row at rowIndex (moves remaining rows to the bottom"
    
    self data insertRow:anArray at:rowIndex.
    self notifyRowsFrom:rowIndex reindexedBy:1.
! !

!SpreadSheet methodsFor:'private'!

reindexListeners
    "returns set of reindex listeners"
    
    reindexListeners ifNil:[ reindexListeners := WeakIdentitySet new. ].
    ^ reindexListeners.
! !

!SpreadSheet methodsFor:'querying'!

columnsCount
    "returns number of columns"
    
    ^ self data columnsCount
!

name
   name ifNil: [^ 'Unnamed'].
   ^ name.
!

name: aString
   aString isString ifFalse: [self error: 'given parameter not string'.].
   name := aString.
!

rowsCount
    "returns number of rows"
    
    ^ self data rowsCount
! !

